@@ -1,6 +1,6 @@
 // std::time_get, std::time_put implementation, GNU version -*- C++ -*-
 
-// Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
+// Copyright (C) 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -117,7 +117,7 @@ namespace std
 	  _M_data->_M_amonth04 = "Apr";
 	  _M_data->_M_amonth05 = "May";
 	  _M_data->_M_amonth06 = "Jun";
-	  _M_data->_M_amonth07 = "July";
+	  _M_data->_M_amonth07 = "Jul";
 	  _M_data->_M_amonth08 = "Aug";
 	  _M_data->_M_amonth09 = "Sep";
 	  _M_data->_M_amonth10 = "Oct";
@@ -265,7 +265,7 @@ namespace std
 	  _M_data->_M_amonth04 = L"Apr";
 	  _M_data->_M_amonth05 = L"May";
 	  _M_data->_M_amonth06 = L"Jun";
-	  _M_data->_M_amonth07 = L"July";
+	  _M_data->_M_amonth07 = L"Jul";
 	  _M_data->_M_amonth08 = L"Aug";
 	  _M_data->_M_amonth09 = L"Sep";
 	  _M_data->_M_amonth10 = L"Oct";
