@@ -22,12 +22,14 @@
 
 #include <testsuite_hooks.h>
 
-// c++/3111
+// libstdc++/7986
 int main()
 {
   using namespace __gnu_test;
 
-  verify_demangle("_Z1fKPFiiE", "f(int (* const)(int))");
+ // cplus-dem FAIL
+  verify_demangle("_Z3fooA30_A_i", "foo(int [30][])");
+
 
   return 0;
 }
